@@ -40,9 +40,12 @@ void Cli7zTest::initTestCase()
 {
     qRegisterMetaType<ArchiveEntry>();
 
-    const auto plugins = KPluginLoader::findPluginsById(QStringLiteral("kerfuffle"), QStringLiteral("kerfuffle_cli7z"));
-    if (plugins.size() == 1) {
-        m_pluginMetadata = plugins.at(0);
+    m_plugin = new Plugin(this);
+    foreach (Plugin *plugin, m_pluginManger.availablePlugins()) {
+        if (plugin->metaData().pluginId() == QStringLiteral("kerfuffle_cli7z")) {
+            m_plugin = plugin;
+            return;
+        }
     }
 }
 
@@ -65,12 +68,12 @@ void Cli7zTest::testArchive_data()
 
 void Cli7zTest::testArchive()
 {
-    if (!m_pluginMetadata.isValid()) {
-        QSKIP("Could not find the cli7z plugin. Skipping test.", SkipSingle);
+    if (!m_plugin->isValid()) {
+        QSKIP("cli7z plugin not available. Skipping test.", SkipSingle);
     }
 
     QFETCH(QString, archivePath);
-    Archive *archive = Archive::create(archivePath, m_pluginMetadata, this);
+    Archive *archive = Archive::create(archivePath, m_plugin, this);
     QVERIFY(archive);
 
     if (!archive->isValid()) {
@@ -97,6 +100,9 @@ void Cli7zTest::testList_data()
 {
     QTest::addColumn<QString>("outputTextFile");
     QTest::addColumn<int>("expectedEntriesCount");
+    QTest::addColumn<bool>("isMultiVolume");
+    // Is zero for non-multi-volume archives:
+    QTest::addColumn<int>("numberOfVolumes");
     // Index of some entry to be tested.
     QTest::addColumn<int>("someEntryIndex");
     // Entry metadata.
@@ -106,34 +112,48 @@ void Cli7zTest::testList_data()
     QTest::addColumn<qulonglong>("expectedSize");
     QTest::addColumn<QString>("expectedTimestamp");
 
+    // p7zip version 16.02 tests
+
+    QTest::newRow("normal-file-1602")
+            << QFINDTESTDATA("data/archive-with-symlink-1602.txt") << 10 << false << 0
+            << 4 << QStringLiteral("testarchive/dir2/file2.txt") << false << false << (qulonglong) 32 << QStringLiteral("2015-05-17T20:41:48");
+
+    QTest::newRow("encrypted-1602")
+            << QFINDTESTDATA("data/archive-encrypted-1602.txt") << 4 << false << 0
+            << 1 << QStringLiteral("file2.txt") << false << true << (qulonglong) 14 << QStringLiteral("2016-03-02T22:37:55");
+
+    QTest::newRow("multi-volume-1602")
+            << QFINDTESTDATA("data/archive-multivol-1602.txt") << 2 << true << 5
+            << 1 << QStringLiteral("largefile2") << false << false << (qulonglong) 2097152 << QStringLiteral("2016-07-17T11:26:19");
+
     // p7zip version 15.14 tests
 
     QTest::newRow("normal-file-1514")
-            << QFINDTESTDATA("data/archive-with-symlink-1514.txt") << 10
+            << QFINDTESTDATA("data/archive-with-symlink-1514.txt") << 10 << false << 0
             << 4 << QStringLiteral("testarchive/dir2/file2.txt") << false << false << (qulonglong) 32 << QStringLiteral("2015-05-17T19:41:48");
 
     QTest::newRow("encrypted-1514")
-            << QFINDTESTDATA("data/archive-encrypted-1514.txt") << 9
+            << QFINDTESTDATA("data/archive-encrypted-1514.txt") << 9 << false << 0
             << 3 << QStringLiteral("testarchive/dir1/file1.txt") << false << true << (qulonglong) 32 << QStringLiteral("2015-05-17T19:41:48");
 
     // p7zip version 15.09 tests
 
     QTest::newRow("normal-file-1509")
-            << QFINDTESTDATA("data/archive-with-symlink-1509.txt") << 10
+            << QFINDTESTDATA("data/archive-with-symlink-1509.txt") << 10 << false << 0
             << 4 << QStringLiteral("testarchive/dir2/file2.txt") << false << false << (qulonglong) 32 << QStringLiteral("2015-05-17T19:41:48");
 
     QTest::newRow("encrypted-1509")
-            << QFINDTESTDATA("data/archive-encrypted-1509.txt") << 9
+            << QFINDTESTDATA("data/archive-encrypted-1509.txt") << 9 << false << 0
             << 3 << QStringLiteral("testarchive/dir1/file1.txt") << false << true << (qulonglong) 32 << QStringLiteral("2015-05-17T19:41:48");
 
     // p7zip version 9.38.1 tests
 
     QTest::newRow("normal-file-9381")
-            << QFINDTESTDATA("data/archive-with-symlink-9381.txt") << 10
+            << QFINDTESTDATA("data/archive-with-symlink-9381.txt") << 10 << false << 0
             << 4 << QStringLiteral("testarchive/dir2/file2.txt") << false << false << (qulonglong) 32 << QStringLiteral("2015-05-17T19:41:48");
 
     QTest::newRow("encrypted-9381")
-            << QFINDTESTDATA("data/archive-encrypted-9381.txt") << 9
+            << QFINDTESTDATA("data/archive-encrypted-9381.txt") << 9 << false << 0
             << 3 << QStringLiteral("testarchive/dir1/file1.txt") << false << true << (qulonglong) 32 << QStringLiteral("2015-05-17T19:41:48");
 }
 
@@ -156,6 +176,12 @@ void Cli7zTest::testList()
 
     QCOMPARE(signalSpy.count(), expectedEntriesCount);
 
+    QFETCH(bool, isMultiVolume);
+    QCOMPARE(plugin->isMultiVolume(), isMultiVolume);
+
+    QFETCH(int, numberOfVolumes);
+    QCOMPARE(plugin->numberOfVolumes(), numberOfVolumes);
+
     QFETCH(int, someEntryIndex);
     QVERIFY(someEntryIndex < signalSpy.count());
     ArchiveEntry entry = qvariant_cast<ArchiveEntry>(signalSpy.at(someEntryIndex).at(0));
@@ -178,6 +204,128 @@ void Cli7zTest::testList()
     plugin->deleteLater();
 }
 
+void Cli7zTest::testListArgs_data()
+{
+    QTest::addColumn<QString>("archiveName");
+    QTest::addColumn<QString>("password");
+    QTest::addColumn<QStringList>("expectedArgs");
+
+    QTest::newRow("unencrypted")
+            << QStringLiteral("/tmp/foo.7z")
+            << QString()
+            << QStringList {
+                   QStringLiteral("l"),
+                   QStringLiteral("-slt"),
+                   QStringLiteral("/tmp/foo.7z")
+               };
+
+    QTest::newRow("header-encrypted")
+            << QStringLiteral("/tmp/foo.7z")
+            << QStringLiteral("1234")
+            << QStringList {
+                   QStringLiteral("l"),
+                   QStringLiteral("-slt"),
+                   QStringLiteral("-p1234"),
+                   QStringLiteral("/tmp/foo.7z")
+               };
+}
+
+void Cli7zTest::testListArgs()
+{
+    QFETCH(QString, archiveName);
+    CliPlugin *plugin = new CliPlugin(this, {QVariant(archiveName)});
+    QVERIFY(plugin);
+
+    const QStringList listArgs = { QStringLiteral("l"),
+                                   QStringLiteral("-slt"),
+                                   QStringLiteral("$PasswordSwitch"),
+                                   QStringLiteral("$Archive") };
+
+    QFETCH(QString, password);
+    const auto replacedArgs = plugin->substituteListVariables(listArgs, password);
+
+    QFETCH(QStringList, expectedArgs);
+    QCOMPARE(replacedArgs, expectedArgs);
+
+    plugin->deleteLater();
+}
+
+void Cli7zTest::testAddArgs_data()
+{
+    QTest::addColumn<QString>("archiveName");
+    QTest::addColumn<QString>("password");
+    QTest::addColumn<bool>("encryptHeader");
+    QTest::addColumn<int>("compressionLevel");
+    QTest::addColumn<ulong>("volumeSize");
+    QTest::addColumn<QStringList>("expectedArgs");
+
+    QTest::newRow("unencrypted")
+            << QStringLiteral("/tmp/foo.7z")
+            << QString() << false << 5 << 0UL
+            << QStringList {
+                   QStringLiteral("a"),
+                   QStringLiteral("/tmp/foo.7z"),
+                   QStringLiteral("-mx=5")
+               };
+
+    QTest::newRow("encrypted")
+            << QStringLiteral("/tmp/foo.7z")
+            << QStringLiteral("1234") << false << 5 << 0UL
+            << QStringList {
+                   QStringLiteral("a"),
+                   QStringLiteral("/tmp/foo.7z"),
+                   QStringLiteral("-p1234"),
+                   QStringLiteral("-mx=5")
+               };
+
+    QTest::newRow("header-encrypted")
+            << QStringLiteral("/tmp/foo.7z")
+            << QStringLiteral("1234") << true << 5 << 0UL
+            << QStringList {
+                   QStringLiteral("a"),
+                   QStringLiteral("/tmp/foo.7z"),
+                   QStringLiteral("-p1234"),
+                   QStringLiteral("-mhe=on"),
+                   QStringLiteral("-mx=5")
+               };
+
+    QTest::newRow("multi-volume")
+            << QStringLiteral("/tmp/foo.7z")
+            << QString() << false << 5 << 2500UL
+            << QStringList {
+                   QStringLiteral("a"),
+                   QStringLiteral("/tmp/foo.7z"),
+                   QStringLiteral("-mx=5"),
+                   QStringLiteral("-v2500k")
+               };
+}
+
+void Cli7zTest::testAddArgs()
+{
+    QFETCH(QString, archiveName);
+    CliPlugin *plugin = new CliPlugin(this, {QVariant(archiveName)});
+    QVERIFY(plugin);
+
+    const QStringList addArgs = { QStringLiteral("a"),
+                                  QStringLiteral("$Archive"),
+                                  QStringLiteral("$PasswordSwitch"),
+                                  QStringLiteral("$CompressionLevelSwitch"),
+                                  QStringLiteral("$MultiVolumeSwitch"),
+                                  QStringLiteral("$Files") };
+
+    QFETCH(QString, password);
+    QFETCH(bool, encryptHeader);
+    QFETCH(int, compressionLevel);
+    QFETCH(ulong, volumeSize);
+
+    QStringList replacedArgs = plugin->substituteAddVariables(addArgs, {}, password, encryptHeader, compressionLevel, volumeSize);
+
+    QFETCH(QStringList, expectedArgs);
+    QCOMPARE(replacedArgs, expectedArgs);
+
+    plugin->deleteLater();
+}
+
 void Cli7zTest::testExtractArgs_data()
 {
     QTest::addColumn<QString>("archiveName");
@@ -260,7 +408,7 @@ void Cli7zTest::testExtractArgs()
     QFETCH(bool, preservePaths);
     QFETCH(QString, password);
 
-    QStringList replacedArgs = plugin->substituteCopyVariables(extractArgs, files, preservePaths, password, QString());
+    QStringList replacedArgs = plugin->substituteCopyVariables(extractArgs, files, preservePaths, password);
     QVERIFY(replacedArgs.size() >= extractArgs.size());
 
     QFETCH(QStringList, expectedArgs);
