@@ -21,7 +21,7 @@
 #define CLIUNARCHIVERTEST_H
 
 #include "cliplugin.h"
-#include <QObject>
+#include "pluginmanager.h"
 
 using namespace Kerfuffle;
 
@@ -36,6 +36,8 @@ private Q_SLOTS:
     void testArchive();
     void testList_data();
     void testList();
+    void testListArgs_data();
+    void testListArgs();
     void testExtraction_data();
     void testExtraction();
     void testExtractArgs_data();
@@ -43,7 +45,8 @@ private Q_SLOTS:
 
 private:
 
-    KPluginMetaData m_pluginMetadata;
+    PluginManager m_pluginManger;
+    Plugin *m_plugin;
 };
 
 Q_DECLARE_METATYPE(ArchiveEntry)
